package com.qmth.distributed.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.distributed.print.business.bean.dto.CourseDimensionImportDto;
import com.qmth.distributed.print.business.entity.CourseDimension;
import com.qmth.distributed.print.business.entity.TeachCourse;
import com.qmth.distributed.print.business.enums.CourseDimensionCodeEnum;
import com.qmth.distributed.print.business.enums.CourseDimensionSourceEnum;
import com.qmth.distributed.print.business.enums.CourseSettingTypeEnum;
import com.qmth.distributed.print.business.mapper.CourseDimensionMapper;
import com.qmth.distributed.print.business.service.BasicExamService;
import com.qmth.distributed.print.business.service.CourseDimensionService;
import com.qmth.distributed.print.business.service.TeachCourseService;
import com.qmth.teachcloud.common.bean.tiku.TikuCourseProperty;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicExam;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.ExceptionResultEnum;
import com.qmth.teachcloud.common.kit.TikuUtils;
import com.qmth.teachcloud.common.util.ExcelUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

/**
 * <p>
 * 课程知识点表 服务实现类
 * </p>
 *
 * @author CaoZixuan
 * @since 2024-02-27
 */
@Service
public class CourseDimensionServiceImpl extends ServiceImpl<CourseDimensionMapper, CourseDimension> implements CourseDimensionService {

    @Resource
    TeachCourseService teachCourseService;

    @Resource
    BasicExamService basicExamService;

    @Resource
    TikuUtils tikuUtils;

    @Transactional
    @Override
    public void importCourseDimension(SysUser requestUser, MultipartFile file, Long examId, String courseCode)
            throws IOException, NoSuchFieldException {
        Long requestUserId = requestUser.getId();
        Long schoolId = requestUser.getSchoolId();

        TeachCourse teachCourse = teachCourseService.findByExamIdCourseCodeAndUserId(examId, courseCode, requestUserId);
        Long teachCourseId = teachCourse.getId();
        // 删除课程设置
        teachCourseService.clearCourseSetting(teachCourseId, CourseSettingTypeEnum.COURSE_DIMENSION);

        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }
        Long semesterId = SystemConstant.convertIdToLong(basicExam.getSemesterId());

        List<String> errorMsgList = new ArrayList<>();
        List<CourseDimensionImportDto> list = new ArrayList<>();
        ExcelUtil.excelReader(file.getInputStream(), Lists.newArrayList(CourseDimensionImportDto.class), (finalExcelList, finalColumnNameList, finalExcelErrorList) -> {
            for (int i = 0; i < finalExcelList.size(); i++) {
                LinkedMultiValueMap<Integer, Object> excelMap = finalExcelList.get(i);
                List<Object> courseDimensionDtoList = excelMap.get(i);
                // 无数据，跳过
                if (CollectionUtils.isEmpty(courseDimensionDtoList)) {
                    continue;
                }
                for (int y = 0; y < courseDimensionDtoList.size(); y++) {

                    // 行索引
                    Integer rowIndex = y + 1;
                    CourseDimensionImportDto courseDimensionImportDto = (CourseDimensionImportDto) courseDimensionDtoList.get(
                            y);
                    String excelCourseCode = StringUtils.trimToNull(courseDimensionImportDto.getCourseCode());
                    String excelCourseName = StringUtils.trimToNull(courseDimensionImportDto.getCourseName());
                    String firstDimensionName = StringUtils.trimToNull(courseDimensionImportDto.getFirstDimensionName());
                    String secondDimensionName = StringUtils.trimToNull(courseDimensionImportDto.getSecondDimensionName());

                    // 本行全部列为空，跳过
                    if (StringUtils.isAllBlank(excelCourseCode, excelCourseName, firstDimensionName, secondDimensionName)) {
                        continue;
                    }
                    if (courseCode.equals(excelCourseCode)) {
                        errorMsgList.add(String.format("第[%s]行,课程代码异常", rowIndex));
                    }
                    list.add(courseDimensionImportDto);
                }
            }
            return finalExcelList;
        }, 2);

        if (CollectionUtils.isNotEmpty(errorMsgList)) {
            throw ExceptionResultEnum.ERROR.exception(String.join(";", errorMsgList));
        }

        // 处理要新增的考察点
        Map<String, List<String>> dimensionMap = new HashMap<>();
        for (CourseDimensionImportDto courseDimensionImportDto : list) {
            String firstDimensionName = courseDimensionImportDto.getFirstDimensionName();
            String secondDimensionName = courseDimensionImportDto.getSecondDimensionName();

            if (dimensionMap.containsKey(firstDimensionName)) {
                List<String> sl = dimensionMap.get(firstDimensionName);
                if (!sl.contains(secondDimensionName)) {
                    sl.add(secondDimensionName);
                    dimensionMap.put(firstDimensionName, sl);
                }
            } else {
                List<String> sl = new ArrayList<>();
                sl.add(secondDimensionName);
                dimensionMap.put(firstDimensionName, sl);
            }
        }
        CourseDimension temp = new CourseDimension();
        temp.setTeachCourseId(teachCourseId);
        temp.setSchoolId(schoolId);
        temp.setSemesterId(semesterId);
        temp.setExamId(examId);
        temp.setCourseCode(courseCode);
        temp.setUserId(requestUserId);
        temp.setSource(CourseDimensionSourceEnum.IMPORT);

        List<CourseDimension> willSaveDimensions = new ArrayList<>();

        AtomicInteger firstNumber = new AtomicInteger(0);
        for (String firstDimensionName : dimensionMap.keySet()) {
            CourseDimension first = new CourseDimension();
            int index = firstNumber.getAndIncrement();
            BeanUtils.copyProperties(temp, first);
            String firstCode = CourseDimensionCodeEnum.getCodeByIndex(index);
            first.setCode(firstCode);
            first.setName(firstDimensionName);
            first.setNumber(index);
            first.insertInfo(requestUserId);
            first.setParentId(0L);
            willSaveDimensions.add(first);

            List<String> secondDimensionList = dimensionMap.get(firstDimensionName);
            Long firstId = first.getId();
            for (int i = 0; i < secondDimensionList.size(); i++) {
                CourseDimension second = new CourseDimension();
                int secondIndex = i + 1;
                BeanUtils.copyProperties(temp, second);
                String secondCode = firstCode + secondIndex;
                second.setCode(secondCode);
                second.setName(secondDimensionList.get(i));
                second.setNumber(secondIndex);
                second.insertInfo(requestUserId);
                second.setParentId(firstId);
                willSaveDimensions.add(second);
            }
        }
        this.saveBatch(willSaveDimensions);
    }

    @Override
    public void syncTikuCourseProperties(Long examId, String courseCode, Long userId) {
        TeachCourse teachCourse = teachCourseService.findByExamIdCourseCodeAndUserId(examId, courseCode, userId);
        Long schoolId = teachCourse.getSchoolId();
        Long teachCourseId = teachCourse.getId();
        BasicExam basicExam = basicExamService.getById(examId);
        if (Objects.isNull(basicExam)) {
            throw ExceptionResultEnum.ERROR.exception("考试不存在");
        }
        Long semesterId = SystemConstant.convertIdToLong(basicExam.getSemesterId());

        if (this.count(new QueryWrapper<CourseDimension>().lambda().eq(CourseDimension::getTeachCourseId, teachCourseId))
                == 0) {
            // 表里没有知识点记录,从题库获取
            List<TikuCourseProperty> tikuDimensionList = tikuUtils.listCourseProperty(teachCourse.getSchoolId(),
                    courseCode);
            if (CollectionUtils.isNotEmpty(tikuDimensionList)) {
                // 删除课程设置
                teachCourseService.clearCourseSetting(teachCourseId, CourseSettingTypeEnum.COURSE_DIMENSION);

                CourseDimension temp = new CourseDimension();
                temp.setTeachCourseId(teachCourseId);
                temp.setSchoolId(schoolId);
                temp.setSemesterId(semesterId);
                temp.setExamId(examId);
                temp.setCourseCode(courseCode);
                temp.setUserId(userId);
                temp.setSource(CourseDimensionSourceEnum.UNION_QUESTION);
                List<CourseDimension> willAddList = new ArrayList<>();

                for (TikuCourseProperty first : tikuDimensionList) {
                    Long fParentId = first.getParentId();
                    if (fParentId > 0) {
                        throw ExceptionResultEnum.ERROR.exception("题库一级知识点父id异常");
                    }
                    String fName = first.getName();
                    String fCode = first.getCode();
                    int fNumber = Integer.parseInt(first.getNumber());

                    CourseDimension willAddFirst = new CourseDimension();
                    BeanUtils.copyProperties(temp, willAddFirst);
                    willAddFirst.setCode(fCode);
                    willAddFirst.setName(fName);
                    willAddFirst.setNumber(fNumber);
                    willAddFirst.insertInfo(userId);
                    willAddFirst.setParentId(0L);
                    willAddList.add(willAddFirst);

                    List<TikuCourseProperty> secondList = first.getPropertyList();
                    for (TikuCourseProperty second : secondList) {
                        Long sParentId = willAddFirst.getId();
                        String sName = second.getName();
                        String sCode = second.getCode();
                        int sNumber = Integer.parseInt(second.getNumber());

                        CourseDimension willAddSecond = new CourseDimension();
                        BeanUtils.copyProperties(temp, willAddSecond);
                        willAddSecond.setCode(sCode);
                        willAddSecond.setName(sName);
                        willAddSecond.setNumber(sNumber);
                        willAddSecond.insertInfo(userId);
                        willAddSecond.setParentId(sParentId);
                        willAddList.add(willAddSecond);
                    }
                }
                if (CollectionUtils.isNotEmpty(willAddList)) {
                    this.saveBatch(willAddList);
                }
            }
        }

    }

    @Override
    public String autoCreateDimensionCode(Long examId, String courseCode, Long parentId) {
        // 一级属性
        boolean isPrimaryProperty = parentId == null || parentId == 0;
        int primaryPropertyNumber;
        int secondaryPropertyNumber;
        String code;
        List<CourseDimension> sameLevelPropertyList = this.list(
                new QueryWrapper<CourseDimension>().lambda().eq(CourseDimension::getExamId, examId).eq(CourseDimension::getCourseCode, courseCode).eq(CourseDimension::getParentId, parentId)
                        .orderByDesc(CourseDimension::getNumber));
        if (sameLevelPropertyList.size() > 0) {
            // 当前级别属性已经有值
            if (isPrimaryProperty) {
                // 一级属性
                primaryPropertyNumber = sameLevelPropertyList.get(0).getNumber() + 1;
                code = CourseDimensionCodeEnum.getCodeByIndex(primaryPropertyNumber);
                if (code == null || code.length() == 0) {
                    CourseDimensionCodeEnum[] questionPropertyCodeEnums = CourseDimensionCodeEnum.values();
                    Set<String> alreadyUsedCodeSet = sameLevelPropertyList.stream().map(CourseDimension::getCode).collect(Collectors.toSet());
                    // 最大编号已经超过属性编号枚举，则查询是否有,没有使用过的编号
                    Set<String> canUseCodeSet = Arrays.stream(questionPropertyCodeEnums)
                            .filter(e -> !alreadyUsedCodeSet.contains(e.getCode()))
                            .sorted(Comparator.comparing(CourseDimensionCodeEnum::getIndex))
                            .map(CourseDimensionCodeEnum::getCode).collect(Collectors.toSet());
                    if (canUseCodeSet.size() == 0) {
                        throw new StatusException("课程属性一级结构编号已经用完");
                    }
                    code = canUseCodeSet.iterator().next();
                }
            } else {
                // 二级属性
                CourseDimension primaryProperty = this.getById(parentId);
                if (Objects.isNull(primaryProperty)) {
                    throw new StatusException("未找到对应的一级属性");
                }
                secondaryPropertyNumber = sameLevelPropertyList.get(0).getNumber() + 1;
                code = primaryProperty.getCode() + secondaryPropertyNumber;
            }
        } else {
            // 当前级别还未建立属性
            if (isPrimaryProperty) {
                // 一级属性
                code = CourseDimensionCodeEnum.A.getCode();
            } else {
                // 二级属性
                CourseDimension primaryProperty = this.getById(parentId);
                if (Objects.isNull(primaryProperty)) {
                    throw new StatusException("未找到对应的一级属性");
                }
                code = primaryProperty.getCode() + 1;
            }
        }
        return code;
    }
}
